class Ticket():
    """
    {
        "externalContent": {
            "content": "eyJSTVZUaWNrZXQiOnsiVGV4dExvbmciOiJFaW56ZWxmYWhydCBLaW5kZXIiLCJUZXh0TWlkZGxlIjoiRWluemVsZmFocnQgS2luZGVyIiwiVGV4dFNob3J0IjoiRWluemVsZmFocnQgS2luZGVyIiwiUmVtYXJrIjoiUmVnZWxwcmVpcyIsIlZhbGlkRnJvbSI6IjI0LjA5LjIwMjIiLCJUYXJpZmZMZXZlbFByaW50IjoiNDAiLCJUYXJpZmZMZXZlbEludGVybiI6IjYwIiwiUHJpY2UiOiI0NDUiLCJUWkNvZGVTdGFydFJlcSI6IjUyMzMiLCJUWkNvZGVTdGFydFJlcyI6IjUyMzMiLCJUWk5hbWVTdGFydCI6IlNjaG1pdHRlbiBLaXJjaGUiLCJWaWFJZCI6IjAiLCJWaWFUZXh0IjoiS8O2bmlnc3RlaW4vSG9oZW1hcmsiLCJUWkNvZGVEZXN0UmVxIjoiNTAwMSIsIlRaQ29kZURlc3RSZXMiOiI1MDAxIiwiVFpOYW1lRGVzdCI6IkZyYW5rZnVydCAoTWFpbikgSGF1cHR3YWNoZSIsIlR5cGVLZXkiOiIzMzAwIiwiU3VidmVudGlvbiI6IjAiLCJUYXJpZmZWZXJzaW9uIjoiNDEiLCJLYXJ0ZW50eXAiOiJLVFlQLUVpbnplbGZhaHJrYXJ0ZSIsIkthcnRlbnR5cEFuemVpZ2VuYW1lIjoiRWluemVsZmFocnQiLCJLQVByb2R1a3RudW1tZXIiOiIxMDE4NiIsIktBSW5mb1RleHQiOiIiLCJWZXJhbnR3b3J0bGljaGVyIjoiUk1WIiwiVmVyYnVlbmRlIjoiUk1WIiwiRXhwb3J0QXR0cmlidXRlIjoiQVRUUi1WVFJCLUVpbnplbHByZWlzPUtpbmR8QVRUUi1WVFJCLXJlcGxTb3J0ZW49MjIwMCAyNTAwIDI4MDAgMzgwMHxBVFRSLUtUWVAtYmlsbGlnVHI9MSIsIkZyZWlnYWJlSWQiOiIyMDczMCIsIkZyZWlnYWJlIjoiMjY0NSwgMzY3MCwgNTAwMSwgNTAwMiwgNTAwMywgNTAwNCwgNTAwNSwgNTAwNiwgNTAxMSwgNTAxMywgNTAxNSwgNTAyMSwgNTAyMiwgNTAyMywgNTAzMSwgNTA0MSwgNTA0MiwgNTA0NiwgNTA1OSwgNTEwMCwgNTIzMywgNjIzOSwgNjYyNiwgNjYzNiwgNjYzNywgNjYzOSwgNjY0OCwgNjY0OSwgNjY1MCwgNjY1NSwgNjY2MCwgNjY2NCwgNjY5MSIsIkZyZWlnYWJlT3ZlcnZpZXciOiJCYWQgSG9tYnVyZyB2LmQuSMO2aGU7QmFkIFNvZGVuIGEuVHMuO0VzY2hib3JuO0ZyaWVkcmljaHNkb3JmO0dsYXNow7x0dGVuIn19",
            "contentType": "LIB_TARIFF"
        },
        "name": "Einzelfahrkarte Regelpreis",
        "desc": "Kinder",
        "price": 445,
        "cur": "EUR",
        "shpCtx": "{\"RMVTicket\":{\"TextLong\":\"Einzelfahrt Kinder\",\"TextMiddle\":\"Einzelfahrt Kinder\",\"TextShort\":\"Einzelfahrt Kinder\",\"Remark\":\"Regelpreis\",\"ValidFrom\":\"24.09.2022\",\"TariffLevelPrint\":\"40\",\"TariffLevelIntern\":\"60\",\"Price\":\"445\",\"TZCodeStartReq\":\"5233\",\"TZCodeStartRes\":\"5233\",\"TZNameStart\":\"Schmitten Kirche\",\"ViaId\":\"0\",\"ViaText\":\"Königstein/Hohemark\",\"TZCodeDestReq\":\"5001\",\"TZCodeDestRes\":\"5001\",\"TZNameDest\":\"Frankfurt (Main) Hauptwache\",\"TypeKey\":\"3300\",\"Subvention\":\"0\",\"TariffVersion\":\"41\",\"Kartentyp\":\"KTYP-Einzelfahrkarte\",\"KartentypAnzeigename\":\"Einzelfahrt\",\"KAProduktnummer\":\"10186\",\"KAInfoText\":\"\",\"Verantwortlicher\":\"RMV\",\"Verbuende\":\"RMV\",\"ExportAttribute\":\"ATTR-VTRB-Einzelpreis=Kind|ATTR-VTRB-replSorten=2200 2500 2800 3800|ATTR-KTYP-billigTr=1\",\"FreigabeId\":\"20730\",\"Freigabe\":\"2645, 3670, 5001, 5002, 5003, 5004, 5005, 5006, 5011, 5013, 5015, 5021, 5022, 5023, 5031, 5041, 5042, 5046, 5059, 5100, 5233, 6239, 6626, 6636, 6637, 6639, 6648, 6649, 6650, 6655, 6660, 6664, 6691\",\"FreigabeOverview\":\"Bad Homburg v.d.Höhe;Bad Soden a.Ts.;Eschborn;Friedrichsdorf;Glashütten\"}}"
    }
    ###############################################
    "shpctx": {
        "RMVTicket":
            {
                "TextLong": "Einzelfahrt Kinder",
                "TextMiddle": "Einzelfahrt Kinder",
                "TextShort": "Einzelfahrt Kinder",
                "Remark": "Regelpreis",
                "ValidFrom": "24.09.2022",
                "TariffLevelPrint": "40",
                "TariffLevelIntern": "60",
                "Price": "445",
                "TZCodeStartReq": "5233",
                "TZCodeStartRes": "5233",
                "TZNameStart": "Schmitten Kirche",
                "ViaId": "0",
                "ViaText": "Königstein/Hohemark",
                "TZCodeDestReq": "5001",
                "TZCodeDestRes": "5001",
                "TZNameDest": "Frankfurt (Main) Hauptwache",
                "TypeKey": "3300",
                "Subvention": "0",
                "TariffVersion": "41",
                "Kartentyp": "KTYP-Einzelfahrkarte",
                "KartentypAnzeigename": "Einzelfahrt",
                "KAProduktnummer": "10186",
                "KAInfoText": "",
                "Verantwortlicher": "RMV",
                "Verbuende": "RMV",
                "ExportAttribute": "ATTR-VTRB-Einzelpreis=Kind|ATTR-VTRB-replSorten=2200 2500 2800 3800|ATTR-KTYP-billigTr=1",
                "FreigabeId": "20730",
                "Freigabe":"2645, 3670, 5001, 5002, 5003, 5004, 5005, 5006, 5011, 5013, 5015, 5021, 5022, 5023, 5031, 5041, 5042, 5046, 5059, 5100, 5233, 6239, 6626, 6636, 6637, 6639, 6648, 6649, 6650, 6655, 6660, 6664, 6691",
                "FreigabeOverview": "Bad Homburg v.d.Höhe;Bad Soden a.Ts.;Eschborn;Friedrichsdorf;Glashütten"
        }        
    }
    """

    def __init__(self) -> None:
        pass